model HareLynxPopulation "Model describing an ecosystem with two populations, hares and lynx, and the interactions between them"
  output Real hare "Hare population";
  output Real lynx "Lynx population";
  SystemDynamics.Levels.Level Hare(x0 = HareInitialPopulation) annotation(Placement(visible = true, transformation(origin = {0, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Rates.Rate_1 Hare_Birth annotation(Placement(visible = true, transformation(origin = {-60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Auxiliary.Gain hare_birth_factor(k = HareBirthFactor) annotation(Placement(visible = true, transformation(origin = {-40, 40}, extent = {{14, -10}, {-14, 10}}, rotation = 0)));
  SystemDynamics.Sources.Source Source1 annotation(Placement(visible = true, transformation(origin = {-100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Auxiliary.Gain hare_density(k = 0.001) annotation(Placement(visible = true, transformation(origin = {-40, 20}, extent = {{-14, -10}, {14, 10}}, rotation = 0)));
  SystemDynamics.Functions.Tabular Hares_Killed_Per_Lynx(x_vals = {0, 500, 1000}, y_vals = {0, 500, 500}) annotation(Placement(visible = true, transformation(origin = {-0, 20}, extent = {{-18, -10}, {18, 10}}, rotation = 0)));
  SystemDynamics.Rates.Rate_1 Hare_Death annotation(Placement(visible = true, transformation(origin = {60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Sources.Sink Sink1 annotation(Placement(visible = true, transformation(origin = {100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Levels.Level Lynx(x0 = LynxInitialPopulation) annotation(Placement(visible = true, transformation(origin = {0, -60}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
  SystemDynamics.Rates.Rate_1 Lynx_Birth annotation(Placement(visible = true, transformation(origin = {-60, -60}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
  SystemDynamics.Auxiliary.Gain lynx_birth_factor(k = LynxBirthFactor) annotation(Placement(visible = true, transformation(origin = {-40, -40}, extent = {{14, -10}, {-14, 10}}, rotation = 0)));
  SystemDynamics.Sources.Source Source2 annotation(Placement(visible = true, transformation(origin = {-100, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Auxiliary.Prod_2 prod_2_1 annotation(Placement(visible = true, transformation(origin = {60, -20}, extent = {{10, 10}, {-10, -10}}, rotation = 0)));
  SystemDynamics.Functions.Tabular Lynx_Death_Fraction(y_vals = {0.94, 0.66, 0.4, 0.35, 0.3, 0.25, 0.2, 0.15, 0.1, 0.07000000000000001, 0.05, 0.05}, x_vals = {0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 1000}) annotation(Placement(visible = true, transformation(origin = {0, -19.945}, extent = {{-18.099, -10.055}, {18.099, 10.055}}, rotation = 0)));
  SystemDynamics.Sources.Sink Sink2 annotation(Placement(visible = true, transformation(origin = {100, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Auxiliary.Prod_2 prod_2_2 annotation(Placement(visible = true, transformation(origin = {60, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  SystemDynamics.Rates.Rate_1 Lynx_Death annotation(Placement(visible = true, transformation(origin = {60, -60}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
  parameter Real HareInitialPopulation = 100000 "Initial level of hare population" annotation(Dialog(group = "Initial Values"));
  parameter Real LynxInitialPopulation = 1500 "Initial level of lynx population" annotation(Dialog(group = "Initial Values"));
  parameter Real HareBirthFactor = 1.25 "Factor of hare birth";
  parameter Real LynxBirthFactor = 0.25 "Factor of lynx birth";
equation
  connect(prod_2_2.y, Hare_Death.u) annotation(Line(visible = true, points = {{-0, -11.25}, {0, 11.25}}, color = {0, 0, 191}, origin = {60, 38.25}));
  connect(prod_2_2.u1, Hares_Killed_Per_Lynx.y) annotation(Line(visible = true, points = {{16.6, 0}, {-16.6, 0}}, color = {0, 0, 191}, origin = {36.4, 20}));
  connect(Lynx_Death_Fraction.u, hare_density.y) annotation(Line(visible = true, points = {{39.521, -19.945}, {34, -19.945}, {34, 20}, {23.8, 20}}, color = {0, 0, 191}, origin = {-54, 0}));
  connect(Source2.MassInPort1, Lynx_Birth.y) annotation(Line(visible = true, points = {{-12, 0}, {12, 0}}, color = {95, 0, 191}, origin = {-77, -60}));
  connect(Lynx_Birth.y1, Lynx.u1) annotation(Line(visible = true, points = {{-22, 0}, {22, 0}}, color = {95, 0, 191}, origin = {-33, -60}));
  connect(lynx_birth_factor.y, Lynx_Birth.u) annotation(Line(visible = true, points = {{-14.8, -50}, {-25, -50}, {-25, -59.5}}, color = {0, 0, 191}, origin = {-35, 10}));
  connect(lynx_birth_factor.u, Lynx.y4) annotation(Line(visible = true, points = {{4.8, -50}, {27.5, -50}, {27.5, -63.5}}, color = {0, 0, 191}, origin = {-35, 10}));
  connect(Sink1.MassInPort1, Hare_Death.y1) annotation(Line(visible = true, points = {{12, 0}, {-12, 0}}, color = {95, 0, 191}, origin = {77, 60}));
  connect(Hare.u2, Hare_Death.y) annotation(Line(visible = true, points = {{-22, 0}, {22, 0}}, color = {95, 0, 191}, origin = {33, 60}));
  connect(hare_density.y, Hares_Killed_Per_Lynx.u) annotation(Line(visible = true, points = {{-7.9, 0}, {7.9, 0}}, color = {0, 0, 191}, origin = {-22.3, 20}));
  connect(Source1.MassInPort1, Hare_Birth.y) annotation(Line(visible = true, points = {{-12, 0}, {12, 0}}, color = {95, 0, 191}, origin = {-77, 60}));
  connect(Hare_Birth.y1, Hare.u1) annotation(Line(visible = true, points = {{-22, 0}, {22, 0}}, color = {95, 0, 191}, origin = {-33, 60}));
  connect(hare_birth_factor.y, Hare_Birth.u) annotation(Line(visible = true, points = {{12.921, 3.167}, {2.721, 3.167}, {2.721, 12.667}}, color = {0, 0, 191}, origin = {-62.721, 36.833}));
  connect(Lynx_Death.u, prod_2_1.y) annotation(Line(visible = true, origin = {60, -38.25}, points = {{0, -11.25}, {0, 11.25}}, color = {0, 0, 127}));
  connect(Lynx_Death.y, Lynx.u2) annotation(Line(visible = true, origin = {33, -60}, points = {{22, 0}, {-22, 0}}, color = {0, 0, 127}));
  connect(Lynx_Death.y1, Sink2.MassInPort1) annotation(Line(visible = true, origin = {77, -60}, points = {{-12, 0}, {12, 0}}, color = {0, 0, 127}));
  connect(Hare.y4, hare_birth_factor.u) annotation(Line(visible = true, origin = {-16.559, 46.75}, points = {{9.059, 6.75}, {9.059, -6.75}, {-13.641, -6.75}}, color = {1, 37, 163}));
  connect(Lynx.y1, prod_2_2.u2) annotation(Line(visible = true, origin = {49.25, -15.167}, points = {{-40.75, -50.333}, {-19.25, -50.333}, {-19.25, 15.167}, {30.75, 15.167}, {30.75, 35.167}, {17.75, 35.167}}, color = {1, 37, 163}));
  connect(Lynx_Death_Fraction.y, prod_2_1.u2) annotation(Line(visible = true, origin = {41.727, -19.973}, points = {{-21.819, 0.027}, {5.273, 0.027}, {5.273, -0.027}, {11.273, -0.027}}, color = {1, 37, 163}));
  connect(Lynx.y2, prod_2_1.u1) annotation(Line(visible = true, origin = {43.795, -37.833}, points = {{-36.295, -15.667}, {-36.295, -2.167}, {36.205, -2.167}, {36.205, 17.833}, {23.205, 17.833}}, color = {1, 37, 163}));
  connect(Hare.y2, hare_density.u) annotation(Line(visible = true, origin = {-22.633, 34.5}, points = {{30.133, 19}, {30.133, -4.5}, {-37.367, -4.5}, {-37.367, -14.5}, {-27.167, -14.5}}, color = {1, 37, 163}));
  hare = Hare.y;
  lynx = Lynx.y;
  annotation(preferredView = "info", experiment(StopTime = 15), Diagram(coordinateSystem(extent = {{-140, -100}, {140, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Text(visible = true, origin = {0, 80}, extent = {{-22.915, -6.626}, {22.915, 6.626}}, textString = "Hare"), Text(visible = true, origin = {0, -80}, extent = {{-22.915, -6.626}, {22.915, 6.626}}, textString = "Lynx")}), uses(Modelica(version = "3.2.3"), SystemDynamics(version = "2.2.1")), __Wolfram(openClass = HareLynxPopulation, PlotSet(plots = {Plot(name = "Hare Vs. Lynx Population", preferred = true, subPlots = {SubPlot(curves = {Curve(x = Hare.level, y = Lynx.level, legend = "Hare Vs. Lynx Population")}, label = Label(x = "Hare Population", y = "Lynx Population"))}), Plot(name = "Population Levels", subPlots = {SubPlot(curves = {Curve(x = time, y = Hare.level, legend = "Hare Population")}), SubPlot(curves = {Curve(x = time, y = Lynx.level, legend = "Lynx Population")}, range = Range(xmin = 0, xmax = 10, ymin = auto, ymax = auto))}), Plot(name = "Birth and Mortality Rates", subPlots = {SubPlot(curves = {Curve(x = time, y = Hare_Birth.rate, legend = "Hare Birth Rate"), Curve(x = time, y = Hare_Death.rate, legend = "Hare Mortality Rate")}), SubPlot(curves = {Curve(x = time, y = Lynx_Birth.rate, legend = "Lynx Birth Rate"), Curve(x = time, y = Lynx_Death.rate, legend = "Lynx Mortality Rate")}, range = Range(xmin = 0, xmax = 10, ymin = auto, ymax = auto))})})), Documentation(info = "<html>


    <p>
This model explores a typical predator/prey system based on Lotka&ndash;Voltera dynamics.  This relatively simple system is a good way to get started with understanding population dynamics. A high density of hares will decrease the death rate of the predator, lynx. A higher density of lynx will in turn increase the death rate of the hares.
</p><p>
<img src=\"modelica://HareLynxPopulation/DocumentationFiles/Diagram.png\" alt=\"Diagram\"/>
</p>
    <h4>Interactive Notebook</h4>
    <p>
The included notebook uses a SystemModelParametricSimulate object to interactively explore the model. With the simulation object, different custom visualizations and interactions can be constructed. In the notebook, a parametric plot of the hare versus lynx population is plotted on top of a locator pane where the initial conditions can be chosen. Additional sliders allow you to change parameter values in the model.
</p><p>
<img src=\"modelica://HareLynxPopulation/DocumentationFiles/Manipulate.png\" alt=\"Manipulate\"/>
</p>

<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://HareLynxPopulation/HareLynxPopulation.nb\">HareLynxPopulation.nb</a>.</p>


</html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}));
end HareLynxPopulation;
